/*
  LiquidCrystal Library - Hello World

 Demonstrates the use a 16x2 LCD display.  The LiquidCrystal
 library works with all LCD displays that are compatible with the
 Hitachi HD44780 driver. There are many of them out there, and you
 can usually tell them by the 16-pin interface.

 This sketch prints "Hello World!" to the LCD
 and shows the time.

  The circuit:
 * LCD RS pin to digital pin 12
 * LCD Enable pin to digital pin 11
 * LCD D4 pin to digital pin 5
 * LCD D5 pin to digital pin 4
 * LCD D6 pin to digital pin 3
 * LCD D7 pin to digital pin 2
 * LCD R/W pin to ground
 * LCD VSS pin to ground
 * LCD VCC pin to 5V
 * 10K resistor:
 * ends to +5V and ground
 * wiper to LCD VO pin (pin 3)

 Library originally added 18 Apr 2008
 by David A. Mellis
 library modified 5 Jul 2009
 by Limor Fried (http://www.ladyada.net)
 example added 9 Jul 2009
 by Tom Igoe
 modified 22 Nov 2010
 by Tom Igoe
 modified 7 Nov 2016
 by Arturo Guadalupi

 This example code is in the public domain.

 http://www.arduino.cc/en/Tutorial/LiquidCrystalHelloWorld

*/

// include the library code:
#include <LiquidCrystal.h>
#include <Servo.h>
//define pin numbers and some constants
#define BuA 6
#define BuB 7
#define BuC 8
//define sample rates and delays, in ms
#define BUTTON_HOLD_DELAY 1000
#define BUTTON_SAMPLE_RATE 10
#define BUTTON_HOLD_RATE 70
#define LOCKED_REFRESH_RATE 100 

const String NumToLet[] = {"0","1","2","3","4","5","6","7","8","9"};

// initialize the library by associating any needed LCD interface pin
// with the arduino pin number it is connected to
const int rs = 12, en = 11, d4 = 5, d5 = 4, d6 = 3, d7 = 2;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);

//keeps track of which screen we're on and if screen is initialized
int endTime = 0;
int screenNum = 0;
bool A_Pressed = false;
bool B_Pressed = false;
bool C_Pressed = false;
bool locked = false;
int cursorX = 0;
int cursorY = 0;
int hms[] = {0,0,0};
Servo servo;
//convets the numer to leters
String numberToLetters(int i){ 
  return NumToLet[i/10] + NumToLet[i % 10];
}
void screen1(){
  if (screenNum != 1){
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("00:00:00 Done?"); 
    lcd.setCursor(0,1);
    lcd.print("A:+ B:- C:Next");
    screenNum = 1;
  }
  else {
    if(C_Pressed){
     cursorX = (cursorX+3) % 12;
     lcd.setCursor(0,1);
     if(cursorX == 9){
      lcd.print("A&B: Set C:Next");
     } else {
      lcd.print("A:+ B:- C:Next ");
      }
     } else if(A_Pressed and B_Pressed){
      if(cursorX == 9) screen2();
     } else if((A_Pressed or B_Pressed) and cursorX != 9){
      lcd.setCursor(cursorX,cursorY);
      hms[cursorX/3] -= 1;
      if (A_Pressed){
        hms[cursorX/3] += 2;
      }
      hms[cursorX/3] = hms[cursorX/3] % 60;
      lcd.print(numberToLetters(hms[cursorX/3]));
    }
}
}
void unlock(){ //Todo:
servo.write(0);               //add servo
}
void lock(){
servo.write(90);
}
void screen2(){
  lock();
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print(numberToLetters(hms[0])+":"+numberToLetters(hms[1])+":"+numberToLetters(hms[2]));
  //:w
  //lcd.print(numbersToLetters(hms[0]) + ":" + numberToLetters(hms[1]) + ":" + numbersToLetters(hms[2]));
  int endtime = int(millis()) + hms[0]*3600000 + hms[1]*60000 + hms[2]*1000;
  int timeleft = 1;
  while (timeleft > 0){
    timeleft = int(endtime)-int(millis());
    const bool updateHMS[] = {hms[0]==timeleft/3600000,hms[1]==timeleft/60000,hms[2]==timeleft/1000};
    for(int i = 0; i < 3; i++){
      if (true) {
        const int factors[] = {3600000,60000,1000};
        hms[i] = timeleft/(factors[i]);
        lcd.setCursor(i*3,0);
        lcd.print(numberToLetters(hms[i]));
      }
      delay(10);

    }
  }
  unlock();
}
void screen3(){
  //unlocking screen. Check to see if possible based on servo library
}
    

void setup() {
  //set up the buttons.
  pinMode(BuA, INPUT);
  pinMode(BuB, INPUT);
  pinMode(BuC, INPUT);
  pinMode(10, INPUT);
  digitalWrite(10, HIGH);
  digitalWrite(BuA,HIGH);
  digitalWrite(BuB,HIGH);
  digitalWrite(BuC,HIGH);

  // set up the LCD's number of columns and rows:
  lcd.begin(16, 2);
  // Print a message to the LCD.
  lcd.print("Welcome!");
  lcd.setCursor(0,1);
  lcd.print("Press to Start");
  Serial.begin(9600);
  while(!Serial){
    delay(1);
  }
  Serial.println("Starting");
  screen0();
  }
bool readButtons(){
  bool abc_state[] = {A_Pressed, B_Pressed, C_Pressed};
  A_Pressed = !bool(digitalRead(BuA));
  B_Pressed = !bool(digitalRead(BuB));
  C_Pressed = !bool(digitalRead(BuC));

  return ((abc_state[0] == A_Pressed) and (abc_state[1] == B_Pressed) and (abc_state[2] == C_Pressed));
}
void screen0(){
  //cur_process = &screen1
  screen1();
}

//set processing function
//void (*cur_process)() = &screen0;
// OR
//define the list of posible screens
void (*SCREEN_LIST[])() = {screen0, screen1, screen2};

void loop() {
  if(readButtons()){
    delay(50);
    (*SCREEN_LIST[screenNum])();
  }


  }
